"""
MTProto Auth Key Generation (Diffie-Hellman key exchange).
Implements the full handshake described at:
https://core.telegram.org/mtproto/auth_key
"""
import os
import time
from hashlib import sha1

from ..crypto import AES, AuthKey, Factorization, rsa
from ..errors import SecurityError
from ..extensions import BinaryReader
from ..helpers import generate_key_data_from_nonce
from ..tl.mtproto_types import (
    ReqPqMultiRequest, ReqDHParamsRequest, SetClientDHParamsRequest,
    PQInnerData, ServerDHParamsOk, ServerDHParamsFail,
    ServerDHInnerData, ClientDHInnerData, DhGenOk, DhGenRetry, DhGenFail,
)


def _get_int(b: bytes, signed: bool = True) -> int:
    return int.from_bytes(b, 'big', signed=signed)


async def do_authentication(sender):
    """
    Execute the 3-step MTProto DH auth key exchange.
    Returns (AuthKey, time_offset_seconds).
    """
    # ── Step 1: ReqPqMulti ──────────────────────────────────────────────
    nonce = int.from_bytes(os.urandom(16), 'big', signed=True)
    res_pq = await sender.send(ReqPqMultiRequest(nonce=nonce))

    if res_pq.nonce != nonce:
        raise SecurityError('Step 1: Invalid nonce from server')

    pq = _get_int(res_pq.pq)
    p, q = Factorization.factorize(pq)
    p_bytes = rsa.get_byte_array(p)
    q_bytes = rsa.get_byte_array(q)
    new_nonce = int.from_bytes(os.urandom(32), 'little', signed=True)

    pq_inner = bytes(PQInnerData(
        pq=rsa.get_byte_array(pq),
        p=p_bytes, q=q_bytes,
        nonce=res_pq.nonce,
        server_nonce=res_pq.server_nonce,
        new_nonce=new_nonce,
    ))

    # ── Step 2: ReqDHParams ─────────────────────────────────────────────
    cipher_text = None
    target_fp = None

    for fp in res_pq.server_public_key_fingerprints:
        ct = rsa.encrypt(fp, pq_inner)
        if ct is not None:
            cipher_text, target_fp = ct, fp
            break

    if cipher_text is None:
        for fp in res_pq.server_public_key_fingerprints:
            ct = rsa.encrypt(fp, pq_inner, use_old=True)
            if ct is not None:
                cipher_text, target_fp = ct, fp
                break

    if cipher_text is None:
        raise SecurityError(
            'Step 2: No matching RSA key for fingerprints: '
            + ', '.join(str(f) for f in res_pq.server_public_key_fingerprints)
        )

    server_dh_params = await sender.send(ReqDHParamsRequest(
        nonce=res_pq.nonce,
        server_nonce=res_pq.server_nonce,
        p=p_bytes, q=q_bytes,
        public_key_fingerprint=target_fp,
        encrypted_data=cipher_text,
    ))

    if server_dh_params.nonce != res_pq.nonce:
        raise SecurityError('Step 2: Invalid nonce from server')
    if server_dh_params.server_nonce != res_pq.server_nonce:
        raise SecurityError('Step 2: Invalid server nonce')

    if isinstance(server_dh_params, ServerDHParamsFail):
        raise SecurityError('Step 2: Server returned DH params fail')

    # ── Step 3: SetClientDHParams ────────────────────────────────────────
    key, iv = generate_key_data_from_nonce(res_pq.server_nonce, new_nonce)

    if len(server_dh_params.encrypted_answer) % 16 != 0:
        raise SecurityError('Step 3: AES block size mismatch')

    plain_answer = AES.decrypt_ige(server_dh_params.encrypted_answer, key, iv)

    with BinaryReader(plain_answer) as reader:
        reader.read(20)  # skip SHA1 hash
        server_dh_inner = reader.tgread_object()

    if not isinstance(server_dh_inner, ServerDHInnerData):
        raise SecurityError(f'Step 3: Expected ServerDHInnerData, got {type(server_dh_inner)}')
    if server_dh_inner.nonce != res_pq.nonce:
        raise SecurityError('Step 3: Invalid nonce in encrypted answer')
    if server_dh_inner.server_nonce != res_pq.server_nonce:
        raise SecurityError('Step 3: Invalid server nonce in encrypted answer')

    dh_prime = _get_int(server_dh_inner.dh_prime, signed=False)
    g = server_dh_inner.g
    g_a = _get_int(server_dh_inner.g_a, signed=False)
    time_offset = server_dh_inner.server_time - int(time.time())

    # Security checks (https://core.telegram.org/mtproto/auth_key)
    safety = 2 ** (2048 - 64)
    if not (1 < g < dh_prime - 1):
        raise SecurityError('g out of range')
    if not (1 < g_a < dh_prime - 1):
        raise SecurityError('g_a out of range')
    if not (safety <= g_a <= dh_prime - safety):
        raise SecurityError('g_a out of safe range')

    b = _get_int(os.urandom(256), signed=False)
    g_b = pow(g, b, dh_prime)
    gab = pow(g_a, b, dh_prime)

    if not (safety <= g_b <= dh_prime - safety):
        raise SecurityError('g_b out of safe range')

    client_dh_inner = bytes(ClientDHInnerData(
        nonce=res_pq.nonce,
        server_nonce=res_pq.server_nonce,
        retry_id=0,
        g_b=rsa.get_byte_array(g_b),
    ))
    client_dh_hashed = sha1(client_dh_inner).digest() + client_dh_inner
    client_dh_encrypted = AES.encrypt_ige(client_dh_hashed, key, iv)

    dh_gen = await sender.send(SetClientDHParamsRequest(
        nonce=res_pq.nonce,
        server_nonce=res_pq.server_nonce,
        encrypted_data=client_dh_encrypted,
    ))

    nonce_types = (DhGenOk, DhGenRetry, DhGenFail)

    if dh_gen.nonce != res_pq.nonce:
        raise SecurityError(f'Step 3: Invalid {type(dh_gen).__name__} nonce')
    if dh_gen.server_nonce != res_pq.server_nonce:
        raise SecurityError(f'Step 3: Invalid {type(dh_gen).__name__} server nonce')

    auth_key = AuthKey(rsa.get_byte_array(gab))
    nonce_number = 1 + nonce_types.index(type(dh_gen))
    expected_hash = auth_key.calc_new_nonce_hash(new_nonce, nonce_number)
    actual_hash = getattr(dh_gen, f'new_nonce_hash{nonce_number}')

    if actual_hash != expected_hash:
        raise SecurityError('Step 3: Invalid new nonce hash')

    if not isinstance(dh_gen, DhGenOk):
        raise AssertionError(f'Step 3 answer was {dh_gen}')

    return auth_key, time_offset



from .connection import Connection, make_connection
from .mtprotosender import MTProtoSender
from .mtprotoplainsender import MTProtoPlainSender
from .mtprotostate import MTProtoState

__all__ = [
    'Connection', 'make_connection',
    'MTProtoSender', 'MTProtoPlainSender', 'MTProtoState',
]



"""
TCP connection with MTProto Abridged transport (simplest & most efficient).
"""
import asyncio
import logging
import struct

__log__ = logging.getLogger(__name__)

# Telegram DC addresses
DC_MAP = {
    1: ('149.154.175.53', 443),
    2: ('149.154.167.51', 443),
    3: ('149.154.175.100', 443),
    4: ('149.154.167.91', 443),
    5: ('91.108.56.130', 443),
}


class Connection:
    """
    Async TCP connection using MTProto Abridged transport.
    The first byte sent is 0xEF to signal abridged mode.
    """

    def __init__(self, ip: str, port: int, *, loggers=None, timeout: float = 10.0):
        self._ip = ip
        self._port = port
        self._timeout = timeout
        self._log = (loggers or {}).get(__name__, __log__)
        self._reader: asyncio.StreamReader = None
        self._writer: asyncio.StreamWriter = None
        self._connected = False

    async def connect(self):
        self._log.info('Connecting to %s:%d (abridged)', self._ip, self._port)
        self._reader, self._writer = await asyncio.wait_for(
            asyncio.open_connection(self._ip, self._port),
            timeout=self._timeout
        )
        # Signal abridged transport
        self._writer.write(b'\xef')
        await self._writer.drain()
        self._connected = True
        self._log.info('Connected')

    async def disconnect(self):
        self._connected = False
        if self._writer:
            try:
                self._writer.close()
                await self._writer.wait_closed()
            except Exception:
                pass
        self._reader = None
        self._writer = None

    async def send(self, data: bytes):
        """Send data wrapped in abridged transport."""
        length = len(data) >> 2
        if length < 0x7f:
            header = bytes([length])
        else:
            header = b'\x7f' + struct.pack('<I', length)[:3]

        self._writer.write(header + data)
        await self._writer.drain()

    async def recv(self) -> bytes:
        """Receive a single MTProto abridged packet."""
        # Read length byte
        first = await self._reader.readexactly(1)
        length = first[0]

        if length == 0x7f:
            # 3-byte length
            raw = await self._reader.readexactly(3)
            length = struct.unpack('<I', raw + b'\x00')[0]

        # length is in 4-byte words
        data = await self._reader.readexactly(length * 4)
        return data

    @property
    def is_connected(self) -> bool:
        return self._connected


class ConnectionTcpFull(Connection):
    """Full transport (includes CRC32). Rarely needed but supported."""

    def __init__(self, ip, port, *, loggers=None, timeout=10.0):
        super().__init__(ip, port, loggers=loggers, timeout=timeout)
        self._send_counter = 0
        self._recv_counter = 0

    async def connect(self):
        self._log.info('Connecting to %s:%d (full)', self._ip, self._port)
        self._reader, self._writer = await asyncio.wait_for(
            asyncio.open_connection(self._ip, self._port),
            timeout=self._timeout
        )
        self._connected = True

    async def send(self, data: bytes):
        import binascii
        payload = struct.pack('<II', len(data) + 12, self._send_counter) + data
        crc = binascii.crc32(payload) & 0xffffffff
        self._writer.write(payload + struct.pack('<I', crc))
        await self._writer.drain()
        self._send_counter += 1

    async def recv(self) -> bytes:
        import binascii
        header = await self._reader.readexactly(8)
        length, seq = struct.unpack('<II', header)
        data = await self._reader.readexactly(length - 12)
        _crc = await self._reader.readexactly(4)
        # CRC check omitted for brevity
        return data


def make_connection(dc_id: int = 2, *, test: bool = False,
                    loggers=None) -> Connection:
    """Create a connection to a Telegram DC."""
    if test:
        test_dc = {
            1: ('149.154.175.10', 443),
            2: ('149.154.167.40', 443),
            3: ('149.154.175.117', 443),
        }
        ip, port = test_dc.get(dc_id, ('149.154.167.40', 443))
    else:
        ip, port = DC_MAP.get(dc_id, ('149.154.167.51', 443))

    return Connection(ip, port, loggers=loggers or {})



"""
BinaryReader - utility to read Telegram's binary protocol data.
All numbers are little-endian as per MTProto spec.
"""
import struct
import time
from datetime import datetime, timedelta, timezone

_EPOCH = datetime(*time.gmtime(0)[:6], tzinfo=timezone.utc)


class BinaryReader:
    """Efficient binary data reader for MTProto protocol."""

    def __init__(self, data: bytes):
        self.stream = data or b''
        self.position = 0
        self._last = None

    # ── Primitive reads ────────────────────────────────────────────────────

    def read_byte(self) -> int:
        val, = struct.unpack_from('<B', self.stream, self.position)
        self.position += 1
        return val

    def read_int(self, signed: bool = True) -> int:
        fmt = '<i' if signed else '<I'
        val, = struct.unpack_from(fmt, self.stream, self.position)
        self.position += 4
        return val

    def read_long(self, signed: bool = True) -> int:
        fmt = '<q' if signed else '<Q'
        val, = struct.unpack_from(fmt, self.stream, self.position)
        self.position += 8
        return val

    def read_float(self) -> float:
        val, = struct.unpack_from('<f', self.stream, self.position)
        self.position += 4
        return val

    def read_double(self) -> float:
        val, = struct.unpack_from('<d', self.stream, self.position)
        self.position += 8
        return val

    def read_large_int(self, bits: int, signed: bool = True) -> int:
        """Read an n-bit integer (int128 / int256)."""
        return int.from_bytes(
            self.read(bits // 8), byteorder='little', signed=signed
        )

    def read(self, length: int = -1) -> bytes:
        if length >= 0:
            result = self.stream[self.position:self.position + length]
            self.position += length
        else:
            result = self.stream[self.position:]
            self.position += len(result)
        if length >= 0 and len(result) != length:
            raise BufferError(
                f'Need {length} bytes, only {len(result)} available; '
                f'last={self._last!r}'
            )
        self._last = result
        return result

    def get_bytes(self) -> bytes:
        return self.stream

    # ── Telegram-encoded reads ─────────────────────────────────────────────

    def tgread_bytes(self) -> bytes:
        """Read a Telegram-encoded byte array."""
        first = self.read_byte()
        if first == 254:
            length = (self.read_byte()
                      | (self.read_byte() << 8)
                      | (self.read_byte() << 16))
            padding = length % 4
        else:
            length = first
            padding = (length + 1) % 4

        data = self.read(length)
        if padding:
            self.read(4 - padding)
        return data

    def tgread_string(self) -> str:
        """Read a Telegram-encoded UTF-8 string."""
        return self.tgread_bytes().decode('utf-8', errors='replace')

    def tgread_bool(self) -> bool:
        val = self.read_int(signed=False)
        if val == 0x997275b5:
            return True
        if val == 0xbc799737:
            return False
        raise RuntimeError(f'Invalid boolean code {val:#010x}')

    def tgread_date(self) -> datetime:
        return _EPOCH + timedelta(seconds=self.read_int())

    def tgread_object(self):
        """Read any TL object by dispatching on its constructor ID."""
        from ..tl.alltlobjects import tlobjects
        from ..tl.core import core_objects

        constructor_id = self.read_int(signed=False)

        # Handle primitives
        if constructor_id == 0x997275b5:
            return True
        if constructor_id == 0xbc799737:
            return False
        if constructor_id == 0x1cb5c415:  # Vector
            count = self.read_int()
            return [self.tgread_object() for _ in range(count)]

        # Look up generated types
        clazz = tlobjects.get(constructor_id)
        if clazz is not None:
            return clazz.from_reader(self)

        # Look up core protocol types
        clazz = core_objects.get(constructor_id)
        if clazz is not None:
            return clazz.from_reader(self)

        # Unknown constructor — Telegram is running a newer layer than our schema.
        # Raise TypeNotFoundError so the caller (mtprotosender) can log at DEBUG
        # level and discard the enclosing message frame cleanly.  The remaining
        # bytes in this reader belong only to the current MTProto message frame,
        # so consuming them here does NOT affect subsequent message frames.
        self.seek(-4)
        from ..errors import TypeNotFoundError
        error = TypeNotFoundError(constructor_id, self.read())
        raise error

    def tgread_vector(self) -> list:
        if self.read_int(signed=False) != 0x1cb5c415:
            raise RuntimeError('Expected vector constructor')
        count = self.read_int()
        return [self.tgread_object() for _ in range(count)]

    # ── Position ───────────────────────────────────────────────────────────

    def tell_position(self) -> int:
        return self.position

    def set_position(self, pos: int):
        self.position = pos

    def seek(self, offset: int):
        self.position += offset

    def close(self):
        self.stream = b''

    # ── Context manager ────────────────────────────────────────────────────

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class CdnFile(TLObject):
    """TL type: upload.cdnFile"""
    CONSTRUCTOR_ID = 0xa99fca4f
    SUBCLASS_OF_ID = 0xf5ccf928

    def __init__(self, bytes: bytes):
        self.bytes = bytes

    def to_dict(self):
        return {
            '_': 'CdnFile',
            'bytes': self.bytes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.bytes))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bytes = reader.tgread_bytes()
        _args['bytes'] = _val_bytes
        return cls(**_args)


class CdnFileReuploadNeeded(TLObject):
    """TL type: upload.cdnFileReuploadNeeded"""
    CONSTRUCTOR_ID = 0xeea8e46e
    SUBCLASS_OF_ID = 0xf5ccf928

    def __init__(self, request_token: bytes):
        self.request_token = request_token

    def to_dict(self):
        return {
            '_': 'CdnFileReuploadNeeded',
            'request_token': self.request_token,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.request_token))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_request_token = reader.tgread_bytes()
        _args['request_token'] = _val_request_token
        return cls(**_args)


class File(TLObject):
    """TL type: upload.file"""
    CONSTRUCTOR_ID = 0x096a18d5
    SUBCLASS_OF_ID = 0x6c9bd728

    def __init__(self, type: 'TypeFileType', mtime: int, bytes: bytes):
        self.type = type
        self.mtime = mtime
        self.bytes = bytes

    def to_dict(self):
        return {
            '_': 'File',
            'type': self.type,
            'mtime': self.mtime,
            'bytes': self.bytes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.type))
        buf.write(struct.pack('<i', self.mtime))
        buf.write(TLObject.serialize_bytes(self.bytes))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_type = reader.tgread_object()
        _args['type'] = _val_type
        _val_mtime = reader.read_int()
        _args['mtime'] = _val_mtime
        _val_bytes = reader.tgread_bytes()
        _args['bytes'] = _val_bytes
        return cls(**_args)


class FileCdnRedirect(TLObject):
    """TL type: upload.fileCdnRedirect"""
    CONSTRUCTOR_ID = 0xf18cda44
    SUBCLASS_OF_ID = 0x6c9bd728

    def __init__(self, dc_id: int, file_token: bytes, encryption_key: bytes, encryption_iv: bytes, file_hashes: List['TypeFileHash']):
        self.dc_id = dc_id
        self.file_token = file_token
        self.encryption_key = encryption_key
        self.encryption_iv = encryption_iv
        self.file_hashes = file_hashes

    def to_dict(self):
        return {
            '_': 'FileCdnRedirect',
            'dc_id': self.dc_id,
            'file_token': self.file_token,
            'encryption_key': self.encryption_key,
            'encryption_iv': self.encryption_iv,
            'file_hashes': self.file_hashes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.dc_id))
        buf.write(TLObject.serialize_bytes(self.file_token))
        buf.write(TLObject.serialize_bytes(self.encryption_key))
        buf.write(TLObject.serialize_bytes(self.encryption_iv))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.file_hashes)))
        for item in self.file_hashes:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_dc_id = reader.read_int()
        _args['dc_id'] = _val_dc_id
        _val_file_token = reader.tgread_bytes()
        _args['file_token'] = _val_file_token
        _val_encryption_key = reader.tgread_bytes()
        _args['encryption_key'] = _val_encryption_key
        _val_encryption_iv = reader.tgread_bytes()
        _args['encryption_iv'] = _val_encryption_iv
        reader.read_int(signed=False)  # skip vector id
        _count_file_hashes = reader.read_int()
        _list_file_hashes = []
        for _ in range(_count_file_hashes):
            _item_file_hashes = reader.tgread_object()
            _list_file_hashes.append(_item_file_hashes)
        _args['file_hashes'] = _list_file_hashes
        return cls(**_args)


class WebFile(TLObject):
    """TL type: upload.webFile"""
    CONSTRUCTOR_ID = 0x21e753bc
    SUBCLASS_OF_ID = 0x68f17f51

    def __init__(self, size: int, mime_type: str, file_type: 'TypeFileType', mtime: int, bytes: bytes):
        self.size = size
        self.mime_type = mime_type
        self.file_type = file_type
        self.mtime = mtime
        self.bytes = bytes

    def to_dict(self):
        return {
            '_': 'WebFile',
            'size': self.size,
            'mime_type': self.mime_type,
            'file_type': self.file_type,
            'mtime': self.mtime,
            'bytes': self.bytes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.size))
        buf.write(TLObject.serialize_bytes(self.mime_type))
        buf.write(bytes(self.file_type))
        buf.write(struct.pack('<i', self.mtime))
        buf.write(TLObject.serialize_bytes(self.bytes))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_size = reader.read_int()
        _args['size'] = _val_size
        _val_mime_type = reader.tgread_string()
        _args['mime_type'] = _val_mime_type
        _val_file_type = reader.tgread_object()
        _args['file_type'] = _val_file_type
        _val_mtime = reader.read_int()
        _args['mtime'] = _val_mtime
        _val_bytes = reader.tgread_bytes()
        _args['bytes'] = _val_bytes
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class SavedMusic(TLObject):
    """TL type: users.savedMusic"""
    CONSTRUCTOR_ID = 0x34a2f297
    SUBCLASS_OF_ID = 0xf813ae37

    def __init__(self, count: int, documents: List['TypeDocument']):
        self.count = count
        self.documents = documents

    def to_dict(self):
        return {
            '_': 'SavedMusic',
            'count': self.count,
            'documents': self.documents,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.documents)))
        for item in self.documents:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_documents = reader.read_int()
        _list_documents = []
        for _ in range(_count_documents):
            _item_documents = reader.tgread_object()
            _list_documents.append(_item_documents)
        _args['documents'] = _list_documents
        return cls(**_args)


class SavedMusicNotModified(TLObject):
    """TL type: users.savedMusicNotModified"""
    CONSTRUCTOR_ID = 0xe3878aa4
    SUBCLASS_OF_ID = 0xf813ae37

    def __init__(self, count: int):
        self.count = count

    def to_dict(self):
        return {
            '_': 'SavedMusicNotModified',
            'count': self.count,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        return cls(**_args)


class UserFull(TLObject):
    """TL type: users.userFull"""
    CONSTRUCTOR_ID = 0x3b6d152e
    SUBCLASS_OF_ID = 0x83df9df5

    def __init__(self, full_user: 'TypeUserFull', chats: List['TypeChat'], users: List['TypeUser']):
        self.full_user = full_user
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'UserFull',
            'full_user': self.full_user,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.full_user))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_full_user = reader.tgread_object()
        _args['full_user'] = _val_full_user
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class Users(TLObject):
    """TL type: users.users"""
    CONSTRUCTOR_ID = 0x62d706b8
    SUBCLASS_OF_ID = 0xf24bf0c0

    def __init__(self, users: List['TypeUser']):
        self.users = users

    def to_dict(self):
        return {
            '_': 'Users',
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class UsersSlice(TLObject):
    """TL type: users.usersSlice"""
    CONSTRUCTOR_ID = 0x315a4974
    SUBCLASS_OF_ID = 0xf24bf0c0

    def __init__(self, count: int, users: List['TypeUser']):
        self.count = count
        self.users = users

    def to_dict(self):
        return {
            '_': 'UsersSlice',
            'count': self.count,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)




"""
AuthKey - wraps the 2048-bit authorization key and provides derived values.
"""
import struct
from hashlib import sha1


class AuthKey:
    """
    Represents the MTProto authorization key (2048-bit).
    Automatically computes aux_hash and key_id from the key bytes.
    """

    def __init__(self, data=None):
        self._key = None
        self.aux_hash = None
        self.key_id = None
        if data is not None:
            self.key = data

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, value):
        if not value:
            self._key = self.aux_hash = self.key_id = None
            return

        if isinstance(value, AuthKey):
            self._key = value._key
            self.aux_hash = value.aux_hash
            self.key_id = value.key_id
            return

        self._key = value
        digest = sha1(self._key).digest()
        # aux_hash = first 8 bytes of SHA1(key) as uint64 LE
        self.aux_hash = struct.unpack_from('<Q', digest, 0)[0]
        # key_id = last 8 bytes of SHA1(key) as uint64 LE
        self.key_id = struct.unpack_from('<Q', digest, 12)[0]

    def calc_new_nonce_hash(self, new_nonce: int, number: int) -> int:
        """
        Calculates new_nonce_hash for DhGen* verification.
        number is 1 for DhGenOk, 2 for DhGenRetry, 3 for DhGenFail.
        """
        nonce_bytes = new_nonce.to_bytes(32, 'little', signed=True)
        data = nonce_bytes + struct.pack('<BQ', number, self.aux_hash)
        return int.from_bytes(sha1(data).digest()[4:20], 'little', signed=True)

    def __bool__(self):
        return bool(self._key)

    def __eq__(self, other):
        return isinstance(other, AuthKey) and other._key == self._key

    def __repr__(self):
        if self._key:
            return f'AuthKey(key_id={self.key_id:#018x})'
        return 'AuthKey(empty)'



from .gzippacked import GzipPacked
from .messagecontainer import MessageContainer
from .rpcresult import RpcResult
from .tlmessage import TLMessage
from ..mtproto_types import HANDSHAKE_TYPES

core_objects = {
    GzipPacked.CONSTRUCTOR_ID: GzipPacked,
    MessageContainer.CONSTRUCTOR_ID: MessageContainer,
    RpcResult.CONSTRUCTOR_ID: RpcResult,
    **HANDSHAKE_TYPES,
}



"""
Base TLObject and TLRequest classes for tglib.
All generated TL types and functions inherit from these.
"""
import base64
import json
import struct
from datetime import datetime, date, timedelta, timezone
import time

_EPOCH_NAIVE = datetime(*time.gmtime(0)[:6])
_EPOCH = _EPOCH_NAIVE.replace(tzinfo=timezone.utc)


def _datetime_to_timestamp(dt):
    if dt.tzinfo is None:
        dt = dt.replace(tzinfo=timezone.utc)
    secs = int((dt - _EPOCH).total_seconds())
    return struct.unpack('i', struct.pack('I', secs & 0xffffffff))[0]


def _json_default(value):
    if isinstance(value, bytes):
        return base64.b64encode(value).decode('ascii')
    elif isinstance(value, datetime):
        return value.isoformat()
    return repr(value)


class TLObject:
    """Base class for all TL types."""
    CONSTRUCTOR_ID = None
    SUBCLASS_OF_ID = None

    @staticmethod
    def serialize_bytes(data) -> bytes:
        """Serialize bytes/str using Telegram's byte encoding."""
        if not isinstance(data, bytes):
            if isinstance(data, str):
                data = data.encode('utf-8')
            else:
                raise TypeError(f'bytes or str expected, not {type(data)}')

        r = []
        if len(data) < 254:
            padding = (len(data) + 1) % 4
            padding = 0 if padding == 0 else 4 - padding
            r.append(bytes([len(data)]))
            r.append(data)
        else:
            padding = len(data) % 4
            padding = 0 if padding == 0 else 4 - padding
            r.append(bytes([
                254,
                len(data) % 256,
                (len(data) >> 8) % 256,
                (len(data) >> 16) % 256,
            ]))
            r.append(data)

        r.append(bytes(padding))
        return b''.join(r)

    @staticmethod
    def serialize_datetime(dt) -> bytes:
        if not dt and not isinstance(dt, timedelta):
            return b'\0\0\0\0'
        if isinstance(dt, datetime):
            dt = _datetime_to_timestamp(dt)
        elif isinstance(dt, date):
            dt = _datetime_to_timestamp(datetime(dt.year, dt.month, dt.day))
        elif isinstance(dt, float):
            dt = int(dt)
        elif isinstance(dt, timedelta):
            dt = _datetime_to_timestamp(datetime.utcnow() + dt)
        if isinstance(dt, int):
            return struct.pack('<i', dt)
        raise TypeError(f'Cannot interpret "{dt}" as a date.')

    @staticmethod
    def pretty_format(obj, indent=None):
        if indent is None:
            if isinstance(obj, TLObject):
                obj = obj.to_dict()
            if isinstance(obj, dict):
                return '{}({})'.format(
                    obj.get('_', 'dict'),
                    ', '.join(f'{k}={TLObject.pretty_format(v)}'
                               for k, v in obj.items() if k != '_')
                )
            elif isinstance(obj, (str, bytes)):
                return repr(obj)
            elif hasattr(obj, '__iter__'):
                return '[{}]'.format(', '.join(TLObject.pretty_format(x) for x in obj))
            return repr(obj)
        else:
            result = []
            if isinstance(obj, TLObject):
                obj = obj.to_dict()
            if isinstance(obj, dict):
                result.append(obj.get('_', 'dict'))
                result.append('(')
                if obj:
                    result.append('\n')
                    indent += 1
                    for k, v in obj.items():
                        if k == '_':
                            continue
                        result.append('\t' * indent)
                        result.append(k)
                        result.append('=')
                        result.append(TLObject.pretty_format(v, indent))
                        result.append(',\n')
                    result.pop()
                    indent -= 1
                    result.append('\n')
                    result.append('\t' * indent)
                result.append(')')
            elif isinstance(obj, (str, bytes)):
                result.append(repr(obj))
            elif hasattr(obj, '__iter__'):
                result.append('[\n')
                indent += 1
                for x in obj:
                    result.append('\t' * indent)
                    result.append(TLObject.pretty_format(x, indent))
                    result.append(',\n')
                indent -= 1
                result.append('\t' * indent)
                result.append(']')
            else:
                result.append(repr(obj))
            return ''.join(result)

    def to_dict(self):
        raise NotImplementedError

    def to_json(self, fp=None, default=_json_default, **kwargs):
        d = self.to_dict()
        if fp:
            return json.dump(d, fp, default=default, **kwargs)
        return json.dumps(d, default=default, **kwargs)

    def __eq__(self, o):
        return isinstance(o, type(self)) and self.to_dict() == o.to_dict()

    def __ne__(self, o):
        return not isinstance(o, type(self)) or self.to_dict() != o.to_dict()

    def __str__(self):
        return TLObject.pretty_format(self)

    def stringify(self):
        return TLObject.pretty_format(self, indent=0)

    def __bytes__(self):
        try:
            return self._bytes()
        except AttributeError:
            raise TypeError('a TLObject was expected but found something else')

    def _bytes(self) -> bytes:
        raise NotImplementedError

    @classmethod
    def from_reader(cls, reader):
        raise NotImplementedError


class TLRequest(TLObject):
    """Represents a content-related TLObject (a request that can be sent)."""

    @staticmethod
    def read_result(reader):
        return reader.tgread_object()

    async def resolve(self, client, utils):
        pass



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class GetCollectibleInfoRequest(TLRequest):
    """TL type: fragment.getCollectibleInfo"""
    CONSTRUCTOR_ID = 0xbe1e85ba
    SUBCLASS_OF_ID = 0xd4ea5790

    def __init__(self, collectible: 'TypeInputCollectible'):
        self.collectible = collectible

    def to_dict(self):
        return {
            '_': 'GetCollectibleInfoRequest',
            'collectible': self.collectible,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.collectible))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_collectible = reader.tgread_object()
        _args['collectible'] = _val_collectible
        return cls(**_args)




"""tglib TL code generator package."""
from .parser import parse_tl, find_layer, TLObject, TLArg
from .generator import generate_tl_modules

__all__ = ['parse_tl', 'find_layer', 'TLObject', 'TLArg', 'generate_tl_modules']



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="tglib",
    version="0.1.0",
    author="Ankit Chaubey",
    author_email="ankitchaubey.dev@gmail.com",
    description="An experimental full MTProto Python client library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ankit-chaubey/TGLib",
    project_urls={
        "Bug Tracker": "https://github.com/ankit-chaubey/TGLib/issues",
        "Source Code": "https://github.com/ankit-chaubey/TGLib",
    },
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Communications :: Chat",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",
    install_requires=[
        "pyaes",
        "pycryptodome",
        "aiosqlite",
    ],
    keywords=["telegram", "mtproto", "tglib", "client", "api", "bot", "async"],
)



"""
AES-IGE encryption/decryption as required by MTProto 2.0.

Will use cryptg (fastest) -> PyCryptodome -> pure Python fallback.
"""
import os
import logging

__log__ = logging.getLogger(__name__)

try:
    import cryptg
    __log__.info('cryptg detected — using C extension for AES-IGE')
    _USE_CRYPTG = True
except ImportError:
    _USE_CRYPTG = False

if not _USE_CRYPTG:
    try:
        from Crypto.Cipher import AES as _PyAES
        _USE_PYCRYPTO = True
        __log__.info('PyCryptodome detected — using it for AES-IGE')
    except ImportError:
        _USE_PYCRYPTO = False

    if not _USE_PYCRYPTO:
        try:
            import pyaes as _pyaes
            _USE_PYAES = True
            __log__.info('pyaes detected — using pure-Python AES fallback')
        except ImportError:
            _USE_PYAES = False
            __log__.warning(
                'No AES backend found! Install cryptg, pycryptodome, or pyaes.'
            )


def _encrypt_ige_python(plain_text: bytes, key: bytes, iv: bytes) -> bytes:
    """Pure Python AES-IGE encryption."""
    # Pad if necessary
    if len(plain_text) % 16:
        plain_text += os.urandom(16 - len(plain_text) % 16)

    aes = _pyaes.AES(key) if _USE_PYAES else None
    iv1, iv2 = bytearray(iv[:16]), bytearray(iv[16:])
    cipher_text = bytearray()
    blocks = len(plain_text) // 16

    for i in range(blocks):
        block = bytearray(plain_text[i * 16:(i + 1) * 16])
        xored = bytes(b ^ iv1[j] for j, b in enumerate(block))
        encrypted = aes.encrypt(xored)
        ct_block = bytes(encrypted[j] ^ iv2[j] for j in range(16))
        cipher_text.extend(ct_block)
        iv1 = bytearray(ct_block)
        iv2 = bytearray(block)

    return bytes(cipher_text)


def _decrypt_ige_python(cipher_text: bytes, key: bytes, iv: bytes) -> bytes:
    """Pure Python AES-IGE decryption."""
    aes = _pyaes.AES(key) if _USE_PYAES else None
    iv1, iv2 = bytearray(iv[:16]), bytearray(iv[16:])
    plain_text = bytearray()
    blocks = len(cipher_text) // 16

    for i in range(blocks):
        ct_block = bytearray(cipher_text[i * 16:(i + 1) * 16])
        xored = bytes(b ^ iv2[j] for j, b in enumerate(ct_block))
        decrypted = aes.decrypt(xored)
        pt_block = bytes(decrypted[j] ^ iv1[j] for j in range(16))
        plain_text.extend(pt_block)
        iv1 = bytearray(ct_block)
        iv2 = bytearray(pt_block)

    return bytes(plain_text)


class AES:
    """AES-IGE cipher interface (MTProto-compatible)."""

    @staticmethod
    def encrypt_ige(plain_text: bytes, key: bytes, iv: bytes) -> bytes:
        if _USE_CRYPTG:
            return cryptg.encrypt_ige(plain_text, key, iv)
        if _USE_PYCRYPTO:
            # PyCryptodome doesn't support IGE natively; use manual
            pass
        return _encrypt_ige_python(plain_text, key, iv)

    @staticmethod
    def decrypt_ige(cipher_text: bytes, key: bytes, iv: bytes) -> bytes:
        if _USE_CRYPTG:
            return cryptg.decrypt_ige(cipher_text, key, iv)
        if _USE_PYCRYPTO:
            pass
        return _decrypt_ige_python(cipher_text, key, iv)

    @staticmethod
    def encrypt_ctr(data: bytes, key: bytes, iv: bytes) -> bytes:
        """CTR mode encryption (used in obfuscated transports)."""
        if _USE_PYCRYPTO:
            ctr = _PyAES.new(key, _PyAES.MODE_CTR, nonce=b'', initial_value=iv)
            return ctr.encrypt(data)
        # Fallback
        if _USE_PYAES:
            counter = _pyaes.Counter(initial_value=int.from_bytes(iv, 'big'))
            cipher = _pyaes.AESModeOfOperationCTR(key, counter=counter)
            return cipher.encrypt(data)
        raise RuntimeError('No AES CTR backend available')

    decrypt_ctr = encrypt_ctr  # CTR is symmetric



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class CheckSearchPostsFloodRequest(TLRequest):
    """TL type: channels.checkSearchPostsFlood"""
    CONSTRUCTOR_ID = 0x22567115
    SUBCLASS_OF_ID = 0xc2c0ccc1

    def __init__(self, query: Optional[str] = None):
        self.query = query

    def to_dict(self):
        return {
            '_': 'CheckSearchPostsFloodRequest',
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.query is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.query is not None:
            buf.write(TLObject.serialize_bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_query = reader.tgread_string()
            _args['query'] = _val_query
        else:
            _args['query'] = None
        return cls(**_args)


class CheckUsernameRequest(TLRequest):
    """TL type: channels.checkUsername"""
    CONSTRUCTOR_ID = 0x10e6bd2c
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', username: str):
        self.channel = channel
        self.username = username

    def to_dict(self):
        return {
            '_': 'CheckUsernameRequest',
            'channel': self.channel,
            'username': self.username,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(TLObject.serialize_bytes(self.username))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_username = reader.tgread_string()
        _args['username'] = _val_username
        return cls(**_args)


class ConvertToGigagroupRequest(TLRequest):
    """TL type: channels.convertToGigagroup"""
    CONSTRUCTOR_ID = 0x0b290c69
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'ConvertToGigagroupRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class CreateChannelRequest(TLRequest):
    """TL type: channels.createChannel"""
    CONSTRUCTOR_ID = 0x91006707
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, title: str, about: str, broadcast: Optional[bool] = None, megagroup: Optional[bool] = None, for_import: Optional[bool] = None, forum: Optional[bool] = None, geo_point: Optional['TypeInputGeoPoint'] = None, address: Optional[str] = None, ttl_period: Optional[int] = None):
        self.title = title
        self.about = about
        self.broadcast = broadcast
        self.megagroup = megagroup
        self.for_import = for_import
        self.forum = forum
        self.geo_point = geo_point
        self.address = address
        self.ttl_period = ttl_period

    def to_dict(self):
        return {
            '_': 'CreateChannelRequest',
            'title': self.title,
            'about': self.about,
            'broadcast': self.broadcast,
            'megagroup': self.megagroup,
            'for_import': self.for_import,
            'forum': self.forum,
            'geo_point': self.geo_point,
            'address': self.address,
            'ttl_period': self.ttl_period,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.broadcast:
            flags |= (1 << 0)
        if self.megagroup:
            flags |= (1 << 1)
        if self.for_import:
            flags |= (1 << 3)
        if self.forum:
            flags |= (1 << 5)
        if self.address is not None:
            flags |= (1 << 2)
        if self.ttl_period is not None:
            flags |= (1 << 4)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(TLObject.serialize_bytes(self.about))
        if self.geo_point is not None:
            buf.write(bytes(self.geo_point))
        if self.address is not None:
            buf.write(TLObject.serialize_bytes(self.address))
        if self.ttl_period is not None:
            buf.write(struct.pack('<i', self.ttl_period))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['broadcast'] = bool(flags & (1 << 0))
        _args['megagroup'] = bool(flags & (1 << 1))
        _args['for_import'] = bool(flags & (1 << 3))
        _args['forum'] = bool(flags & (1 << 5))
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        _val_about = reader.tgread_string()
        _args['about'] = _val_about
        if flags & (1 << 2):
            _val_geo_point = reader.tgread_object()
            _args['geo_point'] = _val_geo_point
        else:
            _args['geo_point'] = None
        if flags & (1 << 2):
            _val_address = reader.tgread_string()
            _args['address'] = _val_address
        else:
            _args['address'] = None
        if flags & (1 << 4):
            _val_ttl_period = reader.read_int()
            _args['ttl_period'] = _val_ttl_period
        else:
            _args['ttl_period'] = None
        return cls(**_args)


class DeactivateAllUsernamesRequest(TLRequest):
    """TL type: channels.deactivateAllUsernames"""
    CONSTRUCTOR_ID = 0x0a245dd3
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'DeactivateAllUsernamesRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class DeleteChannelRequest(TLRequest):
    """TL type: channels.deleteChannel"""
    CONSTRUCTOR_ID = 0xc0111fe3
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'DeleteChannelRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class DeleteHistoryRequest(TLRequest):
    """TL type: channels.deleteHistory"""
    CONSTRUCTOR_ID = 0x9baa9647
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', max_id: int, for_everyone: Optional[bool] = None):
        self.channel = channel
        self.max_id = max_id
        self.for_everyone = for_everyone

    def to_dict(self):
        return {
            '_': 'DeleteHistoryRequest',
            'channel': self.channel,
            'max_id': self.max_id,
            'for_everyone': self.for_everyone,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.for_everyone:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.max_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['for_everyone'] = bool(flags & (1 << 0))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_max_id = reader.read_int()
        _args['max_id'] = _val_max_id
        return cls(**_args)


class DeleteMessagesRequest(TLRequest):
    """TL type: channels.deleteMessages"""
    CONSTRUCTOR_ID = 0x84c1fd4e
    SUBCLASS_OF_ID = 0xced3c06e

    def __init__(self, channel: 'TypeInputChannel', id: List[int]):
        self.channel = channel
        self.id = id

    def to_dict(self):
        return {
            '_': 'DeleteMessagesRequest',
            'channel': self.channel,
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.read_int()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class DeleteParticipantHistoryRequest(TLRequest):
    """TL type: channels.deleteParticipantHistory"""
    CONSTRUCTOR_ID = 0x367544db
    SUBCLASS_OF_ID = 0x2c49c116

    def __init__(self, channel: 'TypeInputChannel', participant: 'TypeInputPeer'):
        self.channel = channel
        self.participant = participant

    def to_dict(self):
        return {
            '_': 'DeleteParticipantHistoryRequest',
            'channel': self.channel,
            'participant': self.participant,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.participant))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        return cls(**_args)


class EditAdminRequest(TLRequest):
    """TL type: channels.editAdmin"""
    CONSTRUCTOR_ID = 0xd33c8902
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', user_id: 'TypeInputUser', admin_rights: 'TypeChatAdminRights', rank: str):
        self.channel = channel
        self.user_id = user_id
        self.admin_rights = admin_rights
        self.rank = rank

    def to_dict(self):
        return {
            '_': 'EditAdminRequest',
            'channel': self.channel,
            'user_id': self.user_id,
            'admin_rights': self.admin_rights,
            'rank': self.rank,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.user_id))
        buf.write(bytes(self.admin_rights))
        buf.write(TLObject.serialize_bytes(self.rank))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_admin_rights = reader.tgread_object()
        _args['admin_rights'] = _val_admin_rights
        _val_rank = reader.tgread_string()
        _args['rank'] = _val_rank
        return cls(**_args)


class EditBannedRequest(TLRequest):
    """TL type: channels.editBanned"""
    CONSTRUCTOR_ID = 0x96e6cd81
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', participant: 'TypeInputPeer', banned_rights: 'TypeChatBannedRights'):
        self.channel = channel
        self.participant = participant
        self.banned_rights = banned_rights

    def to_dict(self):
        return {
            '_': 'EditBannedRequest',
            'channel': self.channel,
            'participant': self.participant,
            'banned_rights': self.banned_rights,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.participant))
        buf.write(bytes(self.banned_rights))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        _val_banned_rights = reader.tgread_object()
        _args['banned_rights'] = _val_banned_rights
        return cls(**_args)


class EditCreatorRequest(TLRequest):
    """TL type: channels.editCreator"""
    CONSTRUCTOR_ID = 0x8f38cd1f
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', user_id: 'TypeInputUser', password: 'TypeInputCheckPasswordSRP'):
        self.channel = channel
        self.user_id = user_id
        self.password = password

    def to_dict(self):
        return {
            '_': 'EditCreatorRequest',
            'channel': self.channel,
            'user_id': self.user_id,
            'password': self.password,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.user_id))
        buf.write(bytes(self.password))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_password = reader.tgread_object()
        _args['password'] = _val_password
        return cls(**_args)


class EditLocationRequest(TLRequest):
    """TL type: channels.editLocation"""
    CONSTRUCTOR_ID = 0x58e63f6d
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', geo_point: 'TypeInputGeoPoint', address: str):
        self.channel = channel
        self.geo_point = geo_point
        self.address = address

    def to_dict(self):
        return {
            '_': 'EditLocationRequest',
            'channel': self.channel,
            'geo_point': self.geo_point,
            'address': self.address,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.geo_point))
        buf.write(TLObject.serialize_bytes(self.address))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_geo_point = reader.tgread_object()
        _args['geo_point'] = _val_geo_point
        _val_address = reader.tgread_string()
        _args['address'] = _val_address
        return cls(**_args)


class EditPhotoRequest(TLRequest):
    """TL type: channels.editPhoto"""
    CONSTRUCTOR_ID = 0xf12e57c9
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', photo: 'TypeInputChatPhoto'):
        self.channel = channel
        self.photo = photo

    def to_dict(self):
        return {
            '_': 'EditPhotoRequest',
            'channel': self.channel,
            'photo': self.photo,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.photo))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_photo = reader.tgread_object()
        _args['photo'] = _val_photo
        return cls(**_args)


class EditTitleRequest(TLRequest):
    """TL type: channels.editTitle"""
    CONSTRUCTOR_ID = 0x566decd0
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', title: str):
        self.channel = channel
        self.title = title

    def to_dict(self):
        return {
            '_': 'EditTitleRequest',
            'channel': self.channel,
            'title': self.title,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(TLObject.serialize_bytes(self.title))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        return cls(**_args)


class ExportMessageLinkRequest(TLRequest):
    """TL type: channels.exportMessageLink"""
    CONSTRUCTOR_ID = 0xe63fadeb
    SUBCLASS_OF_ID = 0xdee644cc

    def __init__(self, channel: 'TypeInputChannel', id: int, grouped: Optional[bool] = None, thread: Optional[bool] = None):
        self.channel = channel
        self.id = id
        self.grouped = grouped
        self.thread = thread

    def to_dict(self):
        return {
            '_': 'ExportMessageLinkRequest',
            'channel': self.channel,
            'id': self.id,
            'grouped': self.grouped,
            'thread': self.thread,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.grouped:
            flags |= (1 << 0)
        if self.thread:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['grouped'] = bool(flags & (1 << 0))
        _args['thread'] = bool(flags & (1 << 1))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_id = reader.read_int()
        _args['id'] = _val_id
        return cls(**_args)


class GetAdminLogRequest(TLRequest):
    """TL type: channels.getAdminLog"""
    CONSTRUCTOR_ID = 0x33ddf480
    SUBCLASS_OF_ID = 0x51f076bc

    def __init__(self, channel: 'TypeInputChannel', q: str, max_id: int, min_id: int, limit: int, events_filter: Optional['TypeChannelAdminLogEventsFilter'] = None, admins: Optional[List['TypeInputUser']] = None):
        self.channel = channel
        self.q = q
        self.max_id = max_id
        self.min_id = min_id
        self.limit = limit
        self.events_filter = events_filter
        self.admins = admins

    def to_dict(self):
        return {
            '_': 'GetAdminLogRequest',
            'channel': self.channel,
            'q': self.q,
            'max_id': self.max_id,
            'min_id': self.min_id,
            'limit': self.limit,
            'events_filter': self.events_filter,
            'admins': self.admins,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.events_filter is not None:
            flags |= (1 << 0)
        if self.admins is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        buf.write(TLObject.serialize_bytes(self.q))
        buf.write(struct.pack('<q', self.max_id))
        buf.write(struct.pack('<q', self.min_id))
        buf.write(struct.pack('<i', self.limit))
        if self.events_filter is not None:
            buf.write(bytes(self.events_filter))
        if self.admins is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.admins)))
            for item in self.admins:
                buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_q = reader.tgread_string()
        _args['q'] = _val_q
        if flags & (1 << 0):
            _val_events_filter = reader.tgread_object()
            _args['events_filter'] = _val_events_filter
        else:
            _args['events_filter'] = None
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_admins = reader.read_int()
            _list_admins = []
            for _ in range(_count_admins):
                _item_admins = reader.tgread_object()
                _list_admins.append(_item_admins)
            _args['admins'] = _list_admins
        else:
            _args['admins'] = None
        _val_max_id = reader.read_long()
        _args['max_id'] = _val_max_id
        _val_min_id = reader.read_long()
        _args['min_id'] = _val_min_id
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetAdminedPublicChannelsRequest(TLRequest):
    """TL type: channels.getAdminedPublicChannels"""
    CONSTRUCTOR_ID = 0xf8b036af
    SUBCLASS_OF_ID = 0x99d5cb14

    def __init__(self, by_location: Optional[bool] = None, check_limit: Optional[bool] = None, for_personal: Optional[bool] = None):
        self.by_location = by_location
        self.check_limit = check_limit
        self.for_personal = for_personal

    def to_dict(self):
        return {
            '_': 'GetAdminedPublicChannelsRequest',
            'by_location': self.by_location,
            'check_limit': self.check_limit,
            'for_personal': self.for_personal,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.by_location:
            flags |= (1 << 0)
        if self.check_limit:
            flags |= (1 << 1)
        if self.for_personal:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['by_location'] = bool(flags & (1 << 0))
        _args['check_limit'] = bool(flags & (1 << 1))
        _args['for_personal'] = bool(flags & (1 << 2))
        return cls(**_args)


class GetChannelRecommendationsRequest(TLRequest):
    """TL type: channels.getChannelRecommendations"""
    CONSTRUCTOR_ID = 0x25a71742
    SUBCLASS_OF_ID = 0x99d5cb14

    def __init__(self, channel: Optional['TypeInputChannel'] = None):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'GetChannelRecommendationsRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.channel is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.channel is not None:
            buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_channel = reader.tgread_object()
            _args['channel'] = _val_channel
        else:
            _args['channel'] = None
        return cls(**_args)


class GetChannelsRequest(TLRequest):
    """TL type: channels.getChannels"""
    CONSTRUCTOR_ID = 0x0a7f6bbb
    SUBCLASS_OF_ID = 0x99d5cb14

    def __init__(self, id: List['TypeInputChannel']):
        self.id = id

    def to_dict(self):
        return {
            '_': 'GetChannelsRequest',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.tgread_object()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class GetFullChannelRequest(TLRequest):
    """TL type: channels.getFullChannel"""
    CONSTRUCTOR_ID = 0x08736a09
    SUBCLASS_OF_ID = 0x225a5109

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'GetFullChannelRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class GetFutureCreatorAfterLeaveRequest(TLRequest):
    """TL type: channels.getFutureCreatorAfterLeave"""
    CONSTRUCTOR_ID = 0xa00918af
    SUBCLASS_OF_ID = 0x2da17977

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'GetFutureCreatorAfterLeaveRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class GetGroupsForDiscussionRequest(TLRequest):
    """TL type: channels.getGroupsForDiscussion"""
    CONSTRUCTOR_ID = 0xf5dad378
    SUBCLASS_OF_ID = 0x99d5cb14

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetGroupsForDiscussionRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetInactiveChannelsRequest(TLRequest):
    """TL type: channels.getInactiveChannels"""
    CONSTRUCTOR_ID = 0x11e831ee
    SUBCLASS_OF_ID = 0x8bf3d7d4

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetInactiveChannelsRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetLeftChannelsRequest(TLRequest):
    """TL type: channels.getLeftChannels"""
    CONSTRUCTOR_ID = 0x8341ecc0
    SUBCLASS_OF_ID = 0x99d5cb14

    def __init__(self, offset: int):
        self.offset = offset

    def to_dict(self):
        return {
            '_': 'GetLeftChannelsRequest',
            'offset': self.offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_offset = reader.read_int()
        _args['offset'] = _val_offset
        return cls(**_args)


class GetMessageAuthorRequest(TLRequest):
    """TL type: channels.getMessageAuthor"""
    CONSTRUCTOR_ID = 0xece2a0e6
    SUBCLASS_OF_ID = 0x2da17977

    def __init__(self, channel: 'TypeInputChannel', id: int):
        self.channel = channel
        self.id = id

    def to_dict(self):
        return {
            '_': 'GetMessageAuthorRequest',
            'channel': self.channel,
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_id = reader.read_int()
        _args['id'] = _val_id
        return cls(**_args)


class GetMessagesRequest(TLRequest):
    """TL type: channels.getMessages"""
    CONSTRUCTOR_ID = 0xad8c9a23
    SUBCLASS_OF_ID = 0xd4b40b5e

    def __init__(self, channel: 'TypeInputChannel', id: List['TypeInputMessage']):
        self.channel = channel
        self.id = id

    def to_dict(self):
        return {
            '_': 'GetMessagesRequest',
            'channel': self.channel,
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.tgread_object()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class GetParticipantRequest(TLRequest):
    """TL type: channels.getParticipant"""
    CONSTRUCTOR_ID = 0xa0ab6cc6
    SUBCLASS_OF_ID = 0x6658151a

    def __init__(self, channel: 'TypeInputChannel', participant: 'TypeInputPeer'):
        self.channel = channel
        self.participant = participant

    def to_dict(self):
        return {
            '_': 'GetParticipantRequest',
            'channel': self.channel,
            'participant': self.participant,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.participant))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        return cls(**_args)


class GetParticipantsRequest(TLRequest):
    """TL type: channels.getParticipants"""
    CONSTRUCTOR_ID = 0x77ced9d0
    SUBCLASS_OF_ID = 0xe60a6e64

    def __init__(self, channel: 'TypeInputChannel', filter: 'TypeChannelParticipantsFilter', offset: int, limit: int, hash: int):
        self.channel = channel
        self.filter = filter
        self.offset = offset
        self.limit = limit
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetParticipantsRequest',
            'channel': self.channel,
            'filter': self.filter,
            'offset': self.offset,
            'limit': self.limit,
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.filter))
        buf.write(struct.pack('<i', self.offset))
        buf.write(struct.pack('<i', self.limit))
        buf.write(struct.pack('<q', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_filter = reader.tgread_object()
        _args['filter'] = _val_filter
        _val_offset = reader.read_int()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetSendAsRequest(TLRequest):
    """TL type: channels.getSendAs"""
    CONSTRUCTOR_ID = 0xe785a43f
    SUBCLASS_OF_ID = 0x38cb8d21

    def __init__(self, peer: 'TypeInputPeer', for_paid_reactions: Optional[bool] = None, for_live_stories: Optional[bool] = None):
        self.peer = peer
        self.for_paid_reactions = for_paid_reactions
        self.for_live_stories = for_live_stories

    def to_dict(self):
        return {
            '_': 'GetSendAsRequest',
            'peer': self.peer,
            'for_paid_reactions': self.for_paid_reactions,
            'for_live_stories': self.for_live_stories,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.for_paid_reactions:
            flags |= (1 << 0)
        if self.for_live_stories:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['for_paid_reactions'] = bool(flags & (1 << 0))
        _args['for_live_stories'] = bool(flags & (1 << 1))
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        return cls(**_args)


class InviteToChannelRequest(TLRequest):
    """TL type: channels.inviteToChannel"""
    CONSTRUCTOR_ID = 0xc9e33d54
    SUBCLASS_OF_ID = 0x3dbe90a1

    def __init__(self, channel: 'TypeInputChannel', users: List['TypeInputUser']):
        self.channel = channel
        self.users = users

    def to_dict(self):
        return {
            '_': 'InviteToChannelRequest',
            'channel': self.channel,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class JoinChannelRequest(TLRequest):
    """TL type: channels.joinChannel"""
    CONSTRUCTOR_ID = 0x24b524c5
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'JoinChannelRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class LeaveChannelRequest(TLRequest):
    """TL type: channels.leaveChannel"""
    CONSTRUCTOR_ID = 0xf836aa95
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel'):
        self.channel = channel

    def to_dict(self):
        return {
            '_': 'LeaveChannelRequest',
            'channel': self.channel,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class ReadHistoryRequest(TLRequest):
    """TL type: channels.readHistory"""
    CONSTRUCTOR_ID = 0xcc104937
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', max_id: int):
        self.channel = channel
        self.max_id = max_id

    def to_dict(self):
        return {
            '_': 'ReadHistoryRequest',
            'channel': self.channel,
            'max_id': self.max_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.max_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_max_id = reader.read_int()
        _args['max_id'] = _val_max_id
        return cls(**_args)


class ReadMessageContentsRequest(TLRequest):
    """TL type: channels.readMessageContents"""
    CONSTRUCTOR_ID = 0xeab5dc38
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', id: List[int]):
        self.channel = channel
        self.id = id

    def to_dict(self):
        return {
            '_': 'ReadMessageContentsRequest',
            'channel': self.channel,
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.read_int()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class ReorderUsernamesRequest(TLRequest):
    """TL type: channels.reorderUsernames"""
    CONSTRUCTOR_ID = 0xb45ced1d
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', order: List[str]):
        self.channel = channel
        self.order = order

    def to_dict(self):
        return {
            '_': 'ReorderUsernamesRequest',
            'channel': self.channel,
            'order': self.order,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.order)))
        for item in self.order:
            buf.write(TLObject.serialize_bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        reader.read_int(signed=False)  # skip vector id
        _count_order = reader.read_int()
        _list_order = []
        for _ in range(_count_order):
            _item_order = reader.tgread_string()
            _list_order.append(_item_order)
        _args['order'] = _list_order
        return cls(**_args)


class ReportAntiSpamFalsePositiveRequest(TLRequest):
    """TL type: channels.reportAntiSpamFalsePositive"""
    CONSTRUCTOR_ID = 0xa850a693
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', msg_id: int):
        self.channel = channel
        self.msg_id = msg_id

    def to_dict(self):
        return {
            '_': 'ReportAntiSpamFalsePositiveRequest',
            'channel': self.channel,
            'msg_id': self.msg_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.msg_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_msg_id = reader.read_int()
        _args['msg_id'] = _val_msg_id
        return cls(**_args)


class ReportSpamRequest(TLRequest):
    """TL type: channels.reportSpam"""
    CONSTRUCTOR_ID = 0xf44a8315
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', participant: 'TypeInputPeer', id: List[int]):
        self.channel = channel
        self.participant = participant
        self.id = id

    def to_dict(self):
        return {
            '_': 'ReportSpamRequest',
            'channel': self.channel,
            'participant': self.participant,
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.participant))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.read_int()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class RestrictSponsoredMessagesRequest(TLRequest):
    """TL type: channels.restrictSponsoredMessages"""
    CONSTRUCTOR_ID = 0x9ae91519
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', restricted: bool):
        self.channel = channel
        self.restricted = restricted

    def to_dict(self):
        return {
            '_': 'RestrictSponsoredMessagesRequest',
            'channel': self.channel,
            'restricted': self.restricted,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.restricted else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_restricted = reader.tgread_bool()
        _args['restricted'] = _val_restricted
        return cls(**_args)


class SearchPostsRequest(TLRequest):
    """TL type: channels.searchPosts"""
    CONSTRUCTOR_ID = 0xf2c4f24d
    SUBCLASS_OF_ID = 0xd4b40b5e

    def __init__(self, offset_rate: int, offset_peer: 'TypeInputPeer', offset_id: int, limit: int, hashtag: Optional[str] = None, query: Optional[str] = None, allow_paid_stars: Optional[int] = None):
        self.offset_rate = offset_rate
        self.offset_peer = offset_peer
        self.offset_id = offset_id
        self.limit = limit
        self.hashtag = hashtag
        self.query = query
        self.allow_paid_stars = allow_paid_stars

    def to_dict(self):
        return {
            '_': 'SearchPostsRequest',
            'offset_rate': self.offset_rate,
            'offset_peer': self.offset_peer,
            'offset_id': self.offset_id,
            'limit': self.limit,
            'hashtag': self.hashtag,
            'query': self.query,
            'allow_paid_stars': self.allow_paid_stars,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.hashtag is not None:
            flags |= (1 << 0)
        if self.query is not None:
            flags |= (1 << 1)
        if self.allow_paid_stars is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.offset_rate))
        buf.write(bytes(self.offset_peer))
        buf.write(struct.pack('<i', self.offset_id))
        buf.write(struct.pack('<i', self.limit))
        if self.hashtag is not None:
            buf.write(TLObject.serialize_bytes(self.hashtag))
        if self.query is not None:
            buf.write(TLObject.serialize_bytes(self.query))
        if self.allow_paid_stars is not None:
            buf.write(struct.pack('<q', self.allow_paid_stars))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_hashtag = reader.tgread_string()
            _args['hashtag'] = _val_hashtag
        else:
            _args['hashtag'] = None
        if flags & (1 << 1):
            _val_query = reader.tgread_string()
            _args['query'] = _val_query
        else:
            _args['query'] = None
        _val_offset_rate = reader.read_int()
        _args['offset_rate'] = _val_offset_rate
        _val_offset_peer = reader.tgread_object()
        _args['offset_peer'] = _val_offset_peer
        _val_offset_id = reader.read_int()
        _args['offset_id'] = _val_offset_id
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        if flags & (1 << 2):
            _val_allow_paid_stars = reader.read_long()
            _args['allow_paid_stars'] = _val_allow_paid_stars
        else:
            _args['allow_paid_stars'] = None
        return cls(**_args)


class SetBoostsToUnblockRestrictionsRequest(TLRequest):
    """TL type: channels.setBoostsToUnblockRestrictions"""
    CONSTRUCTOR_ID = 0xad399cee
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', boosts: int):
        self.channel = channel
        self.boosts = boosts

    def to_dict(self):
        return {
            '_': 'SetBoostsToUnblockRestrictionsRequest',
            'channel': self.channel,
            'boosts': self.boosts,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.boosts))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_boosts = reader.read_int()
        _args['boosts'] = _val_boosts
        return cls(**_args)


class SetDiscussionGroupRequest(TLRequest):
    """TL type: channels.setDiscussionGroup"""
    CONSTRUCTOR_ID = 0x40582bb2
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, broadcast: 'TypeInputChannel', group: 'TypeInputChannel'):
        self.broadcast = broadcast
        self.group = group

    def to_dict(self):
        return {
            '_': 'SetDiscussionGroupRequest',
            'broadcast': self.broadcast,
            'group': self.group,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.broadcast))
        buf.write(bytes(self.group))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_broadcast = reader.tgread_object()
        _args['broadcast'] = _val_broadcast
        _val_group = reader.tgread_object()
        _args['group'] = _val_group
        return cls(**_args)


class SetEmojiStickersRequest(TLRequest):
    """TL type: channels.setEmojiStickers"""
    CONSTRUCTOR_ID = 0x3cd930b7
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', stickerset: 'TypeInputStickerSet'):
        self.channel = channel
        self.stickerset = stickerset

    def to_dict(self):
        return {
            '_': 'SetEmojiStickersRequest',
            'channel': self.channel,
            'stickerset': self.stickerset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.stickerset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_stickerset = reader.tgread_object()
        _args['stickerset'] = _val_stickerset
        return cls(**_args)


class SetMainProfileTabRequest(TLRequest):
    """TL type: channels.setMainProfileTab"""
    CONSTRUCTOR_ID = 0x3583fcb1
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', tab: 'TypeProfileTab'):
        self.channel = channel
        self.tab = tab

    def to_dict(self):
        return {
            '_': 'SetMainProfileTabRequest',
            'channel': self.channel,
            'tab': self.tab,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.tab))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_tab = reader.tgread_object()
        _args['tab'] = _val_tab
        return cls(**_args)


class SetStickersRequest(TLRequest):
    """TL type: channels.setStickers"""
    CONSTRUCTOR_ID = 0xea8ca4f9
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', stickerset: 'TypeInputStickerSet'):
        self.channel = channel
        self.stickerset = stickerset

    def to_dict(self):
        return {
            '_': 'SetStickersRequest',
            'channel': self.channel,
            'stickerset': self.stickerset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.stickerset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_stickerset = reader.tgread_object()
        _args['stickerset'] = _val_stickerset
        return cls(**_args)


class ToggleAntiSpamRequest(TLRequest):
    """TL type: channels.toggleAntiSpam"""
    CONSTRUCTOR_ID = 0x68f3e4eb
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleAntiSpamRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class ToggleAutotranslationRequest(TLRequest):
    """TL type: channels.toggleAutotranslation"""
    CONSTRUCTOR_ID = 0x167fc0a1
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleAutotranslationRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class ToggleForumRequest(TLRequest):
    """TL type: channels.toggleForum"""
    CONSTRUCTOR_ID = 0x3ff75734
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool, tabs: bool):
        self.channel = channel
        self.enabled = enabled
        self.tabs = tabs

    def to_dict(self):
        return {
            '_': 'ToggleForumRequest',
            'channel': self.channel,
            'enabled': self.enabled,
            'tabs': self.tabs,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        buf.write(struct.pack('<I', 0x997275b5 if self.tabs else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        _val_tabs = reader.tgread_bool()
        _args['tabs'] = _val_tabs
        return cls(**_args)


class ToggleJoinRequestRequest(TLRequest):
    """TL type: channels.toggleJoinRequest"""
    CONSTRUCTOR_ID = 0x4c2985b6
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleJoinRequestRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class ToggleJoinToSendRequest(TLRequest):
    """TL type: channels.toggleJoinToSend"""
    CONSTRUCTOR_ID = 0xe4cb9580
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleJoinToSendRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class ToggleParticipantsHiddenRequest(TLRequest):
    """TL type: channels.toggleParticipantsHidden"""
    CONSTRUCTOR_ID = 0x6a6e7854
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleParticipantsHiddenRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class TogglePreHistoryHiddenRequest(TLRequest):
    """TL type: channels.togglePreHistoryHidden"""
    CONSTRUCTOR_ID = 0xeabbb94c
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'TogglePreHistoryHiddenRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class ToggleSignaturesRequest(TLRequest):
    """TL type: channels.toggleSignatures"""
    CONSTRUCTOR_ID = 0x418d549c
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', signatures_enabled: Optional[bool] = None, profiles_enabled: Optional[bool] = None):
        self.channel = channel
        self.signatures_enabled = signatures_enabled
        self.profiles_enabled = profiles_enabled

    def to_dict(self):
        return {
            '_': 'ToggleSignaturesRequest',
            'channel': self.channel,
            'signatures_enabled': self.signatures_enabled,
            'profiles_enabled': self.profiles_enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.signatures_enabled:
            flags |= (1 << 0)
        if self.profiles_enabled:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['signatures_enabled'] = bool(flags & (1 << 0))
        _args['profiles_enabled'] = bool(flags & (1 << 1))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class ToggleSlowModeRequest(TLRequest):
    """TL type: channels.toggleSlowMode"""
    CONSTRUCTOR_ID = 0xedd49ef0
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', seconds: int):
        self.channel = channel
        self.seconds = seconds

    def to_dict(self):
        return {
            '_': 'ToggleSlowModeRequest',
            'channel': self.channel,
            'seconds': self.seconds,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.seconds))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_seconds = reader.read_int()
        _args['seconds'] = _val_seconds
        return cls(**_args)


class ToggleUsernameRequest(TLRequest):
    """TL type: channels.toggleUsername"""
    CONSTRUCTOR_ID = 0x50f24105
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', username: str, active: bool):
        self.channel = channel
        self.username = username
        self.active = active

    def to_dict(self):
        return {
            '_': 'ToggleUsernameRequest',
            'channel': self.channel,
            'username': self.username,
            'active': self.active,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(TLObject.serialize_bytes(self.username))
        buf.write(struct.pack('<I', 0x997275b5 if self.active else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_username = reader.tgread_string()
        _args['username'] = _val_username
        _val_active = reader.tgread_bool()
        _args['active'] = _val_active
        return cls(**_args)


class ToggleViewForumAsMessagesRequest(TLRequest):
    """TL type: channels.toggleViewForumAsMessages"""
    CONSTRUCTOR_ID = 0x9738bb15
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', enabled: bool):
        self.channel = channel
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleViewForumAsMessagesRequest',
            'channel': self.channel,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class UpdateColorRequest(TLRequest):
    """TL type: channels.updateColor"""
    CONSTRUCTOR_ID = 0xd8aa3671
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', for_profile: Optional[bool] = None, color: Optional[int] = None, background_emoji_id: Optional[int] = None):
        self.channel = channel
        self.for_profile = for_profile
        self.color = color
        self.background_emoji_id = background_emoji_id

    def to_dict(self):
        return {
            '_': 'UpdateColorRequest',
            'channel': self.channel,
            'for_profile': self.for_profile,
            'color': self.color,
            'background_emoji_id': self.background_emoji_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.for_profile:
            flags |= (1 << 1)
        if self.color is not None:
            flags |= (1 << 2)
        if self.background_emoji_id is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        if self.color is not None:
            buf.write(struct.pack('<i', self.color))
        if self.background_emoji_id is not None:
            buf.write(struct.pack('<q', self.background_emoji_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['for_profile'] = bool(flags & (1 << 1))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        if flags & (1 << 2):
            _val_color = reader.read_int()
            _args['color'] = _val_color
        else:
            _args['color'] = None
        if flags & (1 << 0):
            _val_background_emoji_id = reader.read_long()
            _args['background_emoji_id'] = _val_background_emoji_id
        else:
            _args['background_emoji_id'] = None
        return cls(**_args)


class UpdateEmojiStatusRequest(TLRequest):
    """TL type: channels.updateEmojiStatus"""
    CONSTRUCTOR_ID = 0xf0d3e6a8
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', emoji_status: 'TypeEmojiStatus'):
        self.channel = channel
        self.emoji_status = emoji_status

    def to_dict(self):
        return {
            '_': 'UpdateEmojiStatusRequest',
            'channel': self.channel,
            'emoji_status': self.emoji_status,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.emoji_status))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_emoji_status = reader.tgread_object()
        _args['emoji_status'] = _val_emoji_status
        return cls(**_args)


class UpdatePaidMessagesPriceRequest(TLRequest):
    """TL type: channels.updatePaidMessagesPrice"""
    CONSTRUCTOR_ID = 0x4b12327b
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, channel: 'TypeInputChannel', send_paid_messages_stars: int, broadcast_messages_allowed: Optional[bool] = None):
        self.channel = channel
        self.send_paid_messages_stars = send_paid_messages_stars
        self.broadcast_messages_allowed = broadcast_messages_allowed

    def to_dict(self):
        return {
            '_': 'UpdatePaidMessagesPriceRequest',
            'channel': self.channel,
            'send_paid_messages_stars': self.send_paid_messages_stars,
            'broadcast_messages_allowed': self.broadcast_messages_allowed,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.broadcast_messages_allowed:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<q', self.send_paid_messages_stars))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['broadcast_messages_allowed'] = bool(flags & (1 << 0))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_send_paid_messages_stars = reader.read_long()
        _args['send_paid_messages_stars'] = _val_send_paid_messages_stars
        return cls(**_args)


class UpdateUsernameRequest(TLRequest):
    """TL type: channels.updateUsername"""
    CONSTRUCTOR_ID = 0x3514b3de
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, channel: 'TypeInputChannel', username: str):
        self.channel = channel
        self.username = username

    def to_dict(self):
        return {
            '_': 'UpdateUsernameRequest',
            'channel': self.channel,
            'username': self.username,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(TLObject.serialize_bytes(self.username))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_username = reader.tgread_string()
        _args['username'] = _val_username
        return cls(**_args)




"""
tglib - A full MTProto 2.0 Python library for Telegram
======================================================

Similar to Telethon and Pyrogram, tglib provides:
  - Full MTProto 2.0 implementation
  - Auth key generation (DH exchange)
  - AES-IGE encrypted transport
  - TL (Type Language) serialization/deserialization
  - High-level client API: send_message, get_me, etc.
  - Raw API access: await client(SomeRequest(...))
  - Session persistence (SQLite)
  - Event/update handling

Quick Start:
    from tglib import TelegramClient
    from tglib.tl import functions, types

    client = TelegramClient('session', api_id, api_hash)

    async def main():
        await client.connect()
        me = await client.get_me()
        await client.send_message('me', 'Hello!')
        await client.disconnect()
"""
from .client import TelegramClient
from .sessions import SQLiteSession, MemorySession
from .errors import (
    TglibError, RPCError, FloodWaitError,
    SessionPasswordNeededError, SecurityError,
)
from . import tl, helpers

__version__ = '0.1.0'
__all__ = [
    'TelegramClient',
    'SQLiteSession',
    'MemorySession',
    'TglibError',
    'RPCError',
    'FloodWaitError',
    'SessionPasswordNeededError',
    'SecurityError',
    'tl',
    'helpers',
]



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class GetCdnFileRequest(TLRequest):
    """TL type: upload.getCdnFile"""
    CONSTRUCTOR_ID = 0x395f69da
    SUBCLASS_OF_ID = 0xf5ccf928

    def __init__(self, file_token: bytes, offset: int, limit: int):
        self.file_token = file_token
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetCdnFileRequest',
            'file_token': self.file_token,
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.file_token))
        buf.write(struct.pack('<q', self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_file_token = reader.tgread_bytes()
        _args['file_token'] = _val_file_token
        _val_offset = reader.read_long()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetCdnFileHashesRequest(TLRequest):
    """TL type: upload.getCdnFileHashes"""
    CONSTRUCTOR_ID = 0x91dc3f31
    SUBCLASS_OF_ID = 0xa5940726

    def __init__(self, file_token: bytes, offset: int):
        self.file_token = file_token
        self.offset = offset

    def to_dict(self):
        return {
            '_': 'GetCdnFileHashesRequest',
            'file_token': self.file_token,
            'offset': self.offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.file_token))
        buf.write(struct.pack('<q', self.offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_file_token = reader.tgread_bytes()
        _args['file_token'] = _val_file_token
        _val_offset = reader.read_long()
        _args['offset'] = _val_offset
        return cls(**_args)


class GetFileRequest(TLRequest):
    """TL type: upload.getFile"""
    CONSTRUCTOR_ID = 0xbe5335be
    SUBCLASS_OF_ID = 0x6c9bd728

    def __init__(self, location: 'TypeInputFileLocation', offset: int, limit: int, precise: Optional[bool] = None, cdn_supported: Optional[bool] = None):
        self.location = location
        self.offset = offset
        self.limit = limit
        self.precise = precise
        self.cdn_supported = cdn_supported

    def to_dict(self):
        return {
            '_': 'GetFileRequest',
            'location': self.location,
            'offset': self.offset,
            'limit': self.limit,
            'precise': self.precise,
            'cdn_supported': self.cdn_supported,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.precise:
            flags |= (1 << 0)
        if self.cdn_supported:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.location))
        buf.write(struct.pack('<q', self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['precise'] = bool(flags & (1 << 0))
        _args['cdn_supported'] = bool(flags & (1 << 1))
        _val_location = reader.tgread_object()
        _args['location'] = _val_location
        _val_offset = reader.read_long()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetFileHashesRequest(TLRequest):
    """TL type: upload.getFileHashes"""
    CONSTRUCTOR_ID = 0x9156982a
    SUBCLASS_OF_ID = 0xa5940726

    def __init__(self, location: 'TypeInputFileLocation', offset: int):
        self.location = location
        self.offset = offset

    def to_dict(self):
        return {
            '_': 'GetFileHashesRequest',
            'location': self.location,
            'offset': self.offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.location))
        buf.write(struct.pack('<q', self.offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_location = reader.tgread_object()
        _args['location'] = _val_location
        _val_offset = reader.read_long()
        _args['offset'] = _val_offset
        return cls(**_args)


class GetWebFileRequest(TLRequest):
    """TL type: upload.getWebFile"""
    CONSTRUCTOR_ID = 0x24e6818d
    SUBCLASS_OF_ID = 0x68f17f51

    def __init__(self, location: 'TypeInputWebFileLocation', offset: int, limit: int):
        self.location = location
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetWebFileRequest',
            'location': self.location,
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.location))
        buf.write(struct.pack('<i', self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_location = reader.tgread_object()
        _args['location'] = _val_location
        _val_offset = reader.read_int()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class ReuploadCdnFileRequest(TLRequest):
    """TL type: upload.reuploadCdnFile"""
    CONSTRUCTOR_ID = 0x9b2754a8
    SUBCLASS_OF_ID = 0xa5940726

    def __init__(self, file_token: bytes, request_token: bytes):
        self.file_token = file_token
        self.request_token = request_token

    def to_dict(self):
        return {
            '_': 'ReuploadCdnFileRequest',
            'file_token': self.file_token,
            'request_token': self.request_token,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.file_token))
        buf.write(TLObject.serialize_bytes(self.request_token))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_file_token = reader.tgread_bytes()
        _args['file_token'] = _val_file_token
        _val_request_token = reader.tgread_bytes()
        _args['request_token'] = _val_request_token
        return cls(**_args)


class SaveBigFilePartRequest(TLRequest):
    """TL type: upload.saveBigFilePart"""
    CONSTRUCTOR_ID = 0xde7b673d
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, file_id: int, file_part: int, file_total_parts: int, bytes: bytes):
        self.file_id = file_id
        self.file_part = file_part
        self.file_total_parts = file_total_parts
        self.bytes = bytes

    def to_dict(self):
        return {
            '_': 'SaveBigFilePartRequest',
            'file_id': self.file_id,
            'file_part': self.file_part,
            'file_total_parts': self.file_total_parts,
            'bytes': self.bytes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.file_id))
        buf.write(struct.pack('<i', self.file_part))
        buf.write(struct.pack('<i', self.file_total_parts))
        buf.write(TLObject.serialize_bytes(self.bytes))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_file_id = reader.read_long()
        _args['file_id'] = _val_file_id
        _val_file_part = reader.read_int()
        _args['file_part'] = _val_file_part
        _val_file_total_parts = reader.read_int()
        _args['file_total_parts'] = _val_file_total_parts
        _val_bytes = reader.tgread_bytes()
        _args['bytes'] = _val_bytes
        return cls(**_args)


class SaveFilePartRequest(TLRequest):
    """TL type: upload.saveFilePart"""
    CONSTRUCTOR_ID = 0xb304a621
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, file_id: int, file_part: int, bytes: bytes):
        self.file_id = file_id
        self.file_part = file_part
        self.bytes = bytes

    def to_dict(self):
        return {
            '_': 'SaveFilePartRequest',
            'file_id': self.file_id,
            'file_part': self.file_part,
            'bytes': self.bytes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.file_id))
        buf.write(struct.pack('<i', self.file_part))
        buf.write(TLObject.serialize_bytes(self.bytes))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_file_id = reader.read_long()
        _args['file_id'] = _val_file_id
        _val_file_part = reader.read_int()
        _args['file_part'] = _val_file_part
        _val_bytes = reader.tgread_bytes()
        _args['bytes'] = _val_bytes
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class BoostsList(TLObject):
    """TL type: premium.boostsList"""
    CONSTRUCTOR_ID = 0x86f8613c
    SUBCLASS_OF_ID = 0x2235a8bd

    def __init__(self, count: int, boosts: List['TypeBoost'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.count = count
        self.boosts = boosts
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'BoostsList',
            'count': self.count,
            'boosts': self.boosts,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.boosts)))
        for item in self.boosts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_boosts = reader.read_int()
        _list_boosts = []
        for _ in range(_count_boosts):
            _item_boosts = reader.tgread_object()
            _list_boosts.append(_item_boosts)
        _args['boosts'] = _list_boosts
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class BoostsStatus(TLObject):
    """TL type: premium.boostsStatus"""
    CONSTRUCTOR_ID = 0x4959427a
    SUBCLASS_OF_ID = 0xc31b1ab9

    def __init__(self, level: int, current_level_boosts: int, boosts: int, boost_url: str, my_boost: Optional[bool] = None, gift_boosts: Optional[int] = None, next_level_boosts: Optional[int] = None, premium_audience: Optional['TypeStatsPercentValue'] = None, prepaid_giveaways: Optional[List['TypePrepaidGiveaway']] = None, my_boost_slots: Optional[List[int]] = None):
        self.level = level
        self.current_level_boosts = current_level_boosts
        self.boosts = boosts
        self.boost_url = boost_url
        self.my_boost = my_boost
        self.gift_boosts = gift_boosts
        self.next_level_boosts = next_level_boosts
        self.premium_audience = premium_audience
        self.prepaid_giveaways = prepaid_giveaways
        self.my_boost_slots = my_boost_slots

    def to_dict(self):
        return {
            '_': 'BoostsStatus',
            'level': self.level,
            'current_level_boosts': self.current_level_boosts,
            'boosts': self.boosts,
            'boost_url': self.boost_url,
            'my_boost': self.my_boost,
            'gift_boosts': self.gift_boosts,
            'next_level_boosts': self.next_level_boosts,
            'premium_audience': self.premium_audience,
            'prepaid_giveaways': self.prepaid_giveaways,
            'my_boost_slots': self.my_boost_slots,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.my_boost_slots is not None:
            flags |= (1 << 2)
        if self.gift_boosts is not None:
            flags |= (1 << 4)
        if self.next_level_boosts is not None:
            flags |= (1 << 0)
        if self.premium_audience is not None:
            flags |= (1 << 1)
        if self.prepaid_giveaways is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.level))
        buf.write(struct.pack('<i', self.current_level_boosts))
        buf.write(struct.pack('<i', self.boosts))
        buf.write(TLObject.serialize_bytes(self.boost_url))
        if self.gift_boosts is not None:
            buf.write(struct.pack('<i', self.gift_boosts))
        if self.next_level_boosts is not None:
            buf.write(struct.pack('<i', self.next_level_boosts))
        if self.premium_audience is not None:
            buf.write(bytes(self.premium_audience))
        if self.prepaid_giveaways is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.prepaid_giveaways)))
            for item in self.prepaid_giveaways:
                buf.write(bytes(self.item))
        if self.my_boost_slots is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.my_boost_slots)))
            for item in self.my_boost_slots:
                buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['my_boost'] = bool(flags & (1 << 2))
        _val_level = reader.read_int()
        _args['level'] = _val_level
        _val_current_level_boosts = reader.read_int()
        _args['current_level_boosts'] = _val_current_level_boosts
        _val_boosts = reader.read_int()
        _args['boosts'] = _val_boosts
        if flags & (1 << 4):
            _val_gift_boosts = reader.read_int()
            _args['gift_boosts'] = _val_gift_boosts
        else:
            _args['gift_boosts'] = None
        if flags & (1 << 0):
            _val_next_level_boosts = reader.read_int()
            _args['next_level_boosts'] = _val_next_level_boosts
        else:
            _args['next_level_boosts'] = None
        if flags & (1 << 1):
            _val_premium_audience = reader.tgread_object()
            _args['premium_audience'] = _val_premium_audience
        else:
            _args['premium_audience'] = None
        _val_boost_url = reader.tgread_string()
        _args['boost_url'] = _val_boost_url
        if flags & (1 << 3):
            reader.read_int(signed=False)  # skip vector id
            _count_prepaid_giveaways = reader.read_int()
            _list_prepaid_giveaways = []
            for _ in range(_count_prepaid_giveaways):
                _item_prepaid_giveaways = reader.tgread_object()
                _list_prepaid_giveaways.append(_item_prepaid_giveaways)
            _args['prepaid_giveaways'] = _list_prepaid_giveaways
        else:
            _args['prepaid_giveaways'] = None
        if flags & (1 << 2):
            reader.read_int(signed=False)  # skip vector id
            _count_my_boost_slots = reader.read_int()
            _list_my_boost_slots = []
            for _ in range(_count_my_boost_slots):
                _item_my_boost_slots = reader.read_int()
                _list_my_boost_slots.append(_item_my_boost_slots)
            _args['my_boost_slots'] = _list_my_boost_slots
        else:
            _args['my_boost_slots'] = None
        return cls(**_args)


class MyBoosts(TLObject):
    """TL type: premium.myBoosts"""
    CONSTRUCTOR_ID = 0x9ae228e2
    SUBCLASS_OF_ID = 0xad3512db

    def __init__(self, my_boosts: List['TypeMyBoost'], chats: List['TypeChat'], users: List['TypeUser']):
        self.my_boosts = my_boosts
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'MyBoosts',
            'my_boosts': self.my_boosts,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.my_boosts)))
        for item in self.my_boosts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_my_boosts = reader.read_int()
        _list_my_boosts = []
        for _ in range(_count_my_boosts):
            _item_my_boosts = reader.tgread_object()
            _list_my_boosts.append(_item_my_boosts)
        _args['my_boosts'] = _list_my_boosts
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)




"""
RSA encryption using Telegram's server public keys.
Used in auth key generation for the initial handshake.
"""
import os
import struct
from hashlib import sha1

try:
    import rsa
    import rsa.core
except ImportError:
    rsa = None

from ..tl.tlobject import TLObject

# {fingerprint: (rsa.PublicKey, is_old)}
_server_keys = {}


def get_byte_array(integer: int) -> bytes:
    """Convert integer to big-endian bytes (as Telegram expects)."""
    return integer.to_bytes(
        (integer.bit_length() + 8 - 1) // 8,
        byteorder='big',
        signed=False
    )


def _compute_fingerprint(key) -> int:
    n = TLObject.serialize_bytes(get_byte_array(key.n))
    e = TLObject.serialize_bytes(get_byte_array(key.e))
    return struct.unpack('<q', sha1(n + e).digest()[-8:])[0]


def add_key(pub: str, *, old: bool = False):
    """Add an RSA public key (PEM format) to the keystore."""
    if rsa is None:
        raise ImportError('Install the "rsa" package: pip install rsa')
    key = rsa.PublicKey.load_pkcs1(pub.encode() if isinstance(pub, str) else pub)
    _server_keys[_compute_fingerprint(key)] = (key, old)


def encrypt(fingerprint: int, data: bytes, *, use_old: bool = False):
    """
    Encrypt data with the server's RSA key identified by fingerprint.
    Returns encrypted bytes, or None if fingerprint not found.
    """
    if rsa is None:
        raise ImportError('Install the "rsa" package: pip install rsa')
    entry = _server_keys.get(fingerprint)
    if entry is None:
        return None
    key, is_old = entry
    if is_old and not use_old:
        return None

    # sha1(data) + data + random_padding → 255 bytes total
    to_encrypt = sha1(data).digest() + data + os.urandom(235 - len(data))
    payload = int.from_bytes(to_encrypt, 'big')
    encrypted = rsa.core.encrypt_int(payload, key.e, key.n)
    return encrypted.to_bytes(256, 'big')


# ── Telegram's production RSA public keys ────────────────────────────────────

_PRODUCTION_KEYS = [
    # New keys (old=False)
    '''-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAruw2yP/BCcsJliRoW5eBVBVle9dtjJw+OYED160Wybum9SXtBBLX
riwt4rROd9csv0t0OHCaTmRqBcQ0J8fxhN6/cpR1GWgOZRUAiQxoMnlt0R93LCX/
j1dnVa/gVbCjdSxpbrfY2g2L4frzjJvdl84Kd9ORYjDEAyFnEA7dD556OptgLQQ2
e2iVNq8NZLYTzLp5YpOdO1doK+ttrltggTCy5SrKeLoCPPbOgGsdxJxyz5KKcZnS
Lj16yE5HvJQn0CNpRdENvRUXe6tBP78O39oJ8BTHp9oIjd6XWXAsp2CvK45Ol8wF
XGF710w9lwCGNbmNxNYhtIkdqfsEcwR5JwIDAQAB
-----END RSA PUBLIC KEY-----''',
    '''-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAvfLHfYH2r9R70w8prHblWt/nDkh+XkgpflqQVcnAfSuTtO05lNPs
pQmL8Y2XjVT4t8cT6xAkdgfmmvnvRPOOKPi0OfJXoRVylFzAQG/j83u5K3kRLbae
7fLccVhKZhY46lvsueI1hQdLgNV9n1cQ3TDS2pQOCtovG4eDl9wacrXOJTG2990V
jgnIKNA0UMoP+KF03qzryqIt3oTvZq03DyWdGK+AZjgBLaDKSnC6qD2cFY81UryR
WOab8zKkWAnhw2kFpcqhI0jdV5QaSCExvnsjVaX0Y1N0870931/5Jb9ICe4nweZ9
kSDF/gip3kWLG0o8XQpChDfyvsqB9OLV/wIDAQAB
-----END RSA PUBLIC KEY-----''',
    '''-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAs/ditzm+mPND6xkhzwFIz6J/968CtkcSE/7Z2qAJiXbmZ3UDJPGr
zqTDHkO30R8VeRM/Kz2f4nR05GIFiITl4bEjvpy7xqRDspJcCFIOcyXm8abVDhF+
th6knSU0yLtNKuQVP6voMrnt9MV1X92LGZQLgdHZbPQz0Z5qIpaKhdyA8DEvWWvS
Uwwc+yi1/gGaybwlzZwqXYoPOhwMebzKUk0xW14htcJrRrq+PXXQbRzTMynseCoP
Ioke0dtCodbA3qQxQovE16q9zz4Otv2k4j63cz53J+mhkVWAeWxVGI0lltJmWtEY
K6er8VqqWot3nqmWMXogrgRLggv/NbbooQIDAQAB
-----END RSA PUBLIC KEY-----''',
    '''-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAvmpxVY7ld/8DAjz6F6q05shjg8/4p6047bn6/m8yPy1RBsvIyvuD
uGnP/RzPEhzXQ9UJ5Ynmh2XJZgHoE9xbnfxL5BXHplJhMtADXKM9bWB11PU1Eioc
3+AXBB8QiNFBn2XI5UkO5hPhbb9mJpjA9Uhw8EdfqJP8QetVsI/xrCEbwEXe0xvi
fRLJbY08/Gp66KpQvy7g8w7VB8wlgePexW3pT13Ap6vuC+mQuJPyiHvSxjEKHgqe
Pji9NP3tJUFQjcECqcm0yV7/2d0t/pbCm+ZH1sadZspQCEPPrtbkQBlvHb4OLiIW
PGHKSMeRFvp3IWcmdJqXahxLCUS1Eh6MAQIDAQAB
-----END RSA PUBLIC KEY-----''',
]

_OLD_KEYS = [
    '''-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAwVACPi9w23mF3tBkdZz+zwrzKOaaQdr01vAbU4E1pvkfj4sqDsm6
lyDONS789sVoD/xCS9Y0hkkC3gtL1tSfTlgCMOOul9lcixlEKzwKENj1Yz/s7daS
an9tqw3bfUV/nqgbhGX81v/+7RFAEd+RwFnK7a+XYl9sluzHRyVVaTTveB2GazTw
Efzk2DWgkBluml8OREmvfraX3bkHZJTKX4EQSjBbbdJ2ZXIsRrYOXfaA+xayEGB+
8hdlLmAjbCVfaigxX0CDqWeR1yFL9kwd9P0NsZRPsmoqVwMbMu7mStFai6aIhc3n
Slv8kg9qv1m6XHVQY3PnEw+QQtqSIXklHwIDAQAB
-----END RSA PUBLIC KEY-----''',
    '''-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAxq7aeLAqJR20tkQQMfRn+ocfrtMlJsQ2Uksfs7Xcoo77jAid0bRt
ksiVmT2HEIJUlRxfABoPBV8wY9zRTUMaMA654pUX41mhyVN+XoerGxFvrs9dF1Ru
vCHbI02dM2ppPvyytvvMoefRoL5BTcpAihFgm5xCaakgsJ/tH5oVl74CdhQw8J5L
xI/K++KJBUyZ26Uba1632cOiq05JBUW0Z2vWIOk4BLysk7+U9z+SxynKiZR3/xdi
XvFKk01R3BHV+GUKM2RYazpS/P8v7eyKhAbKxOdRcFpHLlVwfjyM1VlDQrEZxsMp
NTLYXb6Sce1Uov0YtNx5wEowlREH1WOTlwIDAQAB
-----END RSA PUBLIC KEY-----''',
]

if rsa is not None:
    for _pub in _PRODUCTION_KEYS:
        add_key(_pub, old=False)
    for _pub in _OLD_KEYS:
        add_key(_pub, old=True)
